% One dimensional quantum mechanical scattering of a Gaussian wave-packet
% at a potentialstep.
% Main script:
% Basic calculation is done here:
%    Definition of parameters, vectors of sampling points; calculation of the
%    potential and the startfunction; memory preallocation for arrays containing
%    the results; execution of the time evolution loop; calculating the
%    normalization factor of the momentum-wavefunction; 
%    calculating coefficients of transition and reflection;
%    writing a structure array containing the data necessary to
%    perform the animation; calling the animation routine;
%
% The natural units system is used, as listed on the following site:
%        http://physics.nist.gov/cgi-bin/cuu/Results?search_for=natural

%clear  all
%close all

function animation_data = Quantenstreuung1D()

% get guidata
qs_main = findobj(0,'Tag','QS_MAIN');
myhandles = guidata(qs_main);

h_bar = 1;
mass  = 1;

grid_range       = 20;  % sampling points are between -grid_range and +grid_range 
nr_of_gridpoints = 256;         % number of sampling points
step_width       = 2*grid_range / nr_of_gridpoints;     % increment in the configuration space

kinetic_energy        = myhandles.kinenergy;                 % kinetic energy of the wave packet (expectation value at t=0)
potential_step_height = myhandles.potenergy;                 % height of potential step
energy_relation       = kinetic_energy / potential_step_height;

wavenumber_t0 = sqrt(2*mass * kinetic_energy / h_bar^2);  % wavenumber in relation to kinetic energy
position_t0 = -5;                                        % starting position of wave packet(expactation value at t=0)
t0 = 0;

% vector of sampling-points
x = -grid_range:step_width:grid_range-step_width;
				    

% evaluating the vector of wavenumbers
delta_wavenumber = 2*pi / (nr_of_gridpoints * step_width);
wavenumber_left  = 0:delta_wavenumber:(nr_of_gridpoints/2-1)*delta_wavenumber;
wavenumber_right = -(nr_of_gridpoints / 2*delta_wavenumber):delta_wavenumber:-1*delta_wavenumber;
wavenumber       = [wavenumber_left , wavenumber_right];
wavenumber_control = max(wavenumber);

% checking if the wavenumber corresponding to the kinetic energy lies
% within the nyquist limit
if wavenumber_t0 > wavenumber_control 
  error('The kinetic energy value is to high; wavnumber_t0 > nyquist')
end

% evaluation of the potential step
potential_left  = zeros(1,nr_of_gridpoints/2);
potential_right = (potential_step_height)*ones(1,nr_of_gridpoints/2);
potential       = [potential_left , potential_right]; 

%operator of kinetic energy (in fourier space)
kinetic_energy_operator = (h_bar^2.*wavenumber.^2) ./ (2*mass);

%start function (Gaussian):
width = 1;
wavefunction_0 = (2*pi*width^2)^(-1/4)*exp((1i*wavenumber_t0*x)-(x-position_t0).^2/(4*width^2));

%increment of time-evolution
time_step = 5e-3;
nr_of_timesteps = 500;

%preallocating memory for the matrices which will contain the results
wavefunction            = zeros(nr_of_timesteps + 1, nr_of_gridpoints); %Matrix containing the wavefunction at a certain timein every row
abs_val_wavefun         = zeros(nr_of_timesteps + 1, nr_of_gridpoints); %squared absolute value of the wavefunction (probability density)
abs_val_momentumwavefun = zeros(nr_of_timesteps + 1, nr_of_gridpoints); %squared absolute value of the wavefunction in fourier space

wavefunction(1,:)          = wavefunction_0; 
abs_val_wavefun(1,:)       = wavefunction_0.*conj(wavefunction_0);     
abs_val_momentumwavefun(1,:) = fft(wavefunction_0).*conj(fft(wavefunction_0)); 



                     %%%%%%%%%%%%%%%%%%%%__Time evolution__%%%%%%%%%%%%%%%%%%%%%%%


% Evolution factor potential
expV = exp(-1i*potential*time_step/(2*h_bar));
% Evolution factor kinetic energy
expT = exp(-1i*kinetic_energy_operator*time_step/h_bar);

inc = 0;
for t = 0:time_step:nr_of_timesteps * time_step;     % for-loop performs time evolution
    inc = inc + 1;

    % applying the potential operator
    phi_x_0 = expV.*wavefunction_0;
    
    % transformation into k-space
    phi_k_0 = fft(phi_x_0);
    
    % applying kinetic operator in k-space
    phi_k_1 = expT.*phi_k_0;
    
    % back transformation into configuration space
    phi_x_1 = ifft(phi_k_1);
   
    % applying the potential operator again
    wavefunction_1 = expV.*phi_x_1;

    % implicit midpoint rule
    %k = h_bar^2/(2*mass);
    %H_matrix = diag(2*k + potential) + k*diag(-ones(1,nr_of_gridpoints-1),-1) + k*diag(-ones(1,nr_of_gridpoints-1),1);
    %wavefunction_1 = (1 - 1/2*1i*time_step*(H_matrix))/(1 + 1/2*1i*time_step*(H_matrix)) * wavefunction_0';
    
    psi_k_t = fft(wavefunction_1);

    wavefunction(inc+1,:) = wavefunction_1;
    abs_val_wavefun(inc+1,:) = conj(wavefunction_1).*wavefunction_1;
    abs_val_momentumwavefun(inc+1,:) = conj(psi_k_t).*psi_k_t;
    
    wavefunction_0 = wavefunction_1;
end

% scale factor for the momentum-space wavefunction; necessary to evaluate
% the momentum corresponding to the vector of wavenumbers. The
% wavefunction in momentum-space will be plotted over the momentum.
norm_fact = sqrt(sum(abs_val_momentumwavefun(1,:)*delta_wavenumber));
abs_val_momentumwavefun = abs_val_momentumwavefun / norm_fact^2;

% evaluation of coefficients of reflection and transition
reflection_coeff = sum(abs_val_wavefun(end,1:nr_of_gridpoints/2)*step_width);                   % area under the left part of the wavefunction (reflected part)
transition_coeff  = sum(abs_val_wavefun(end,nr_of_gridpoints/2+1: nr_of_gridpoints)*step_width);% area under the right part (transmitted part)


% writing a structure array; input of play_qm1D.m
% play_qm1D.m performs the animation;
animation_data.abs_val_wavefun         = abs_val_wavefun;
animation_data.abs_val_momentumwavefun = [abs_val_momentumwavefun(:, nr_of_gridpoints/2+1:end),abs_val_momentumwavefun(:,1:nr_of_gridpoints/2)];
animation_data.energy_relation         = energy_relation;
animation_data.potential               = potential;
animation_data.x                       = x;
animation_data.momentum                = h_bar*[wavenumber(nr_of_gridpoints/2+1:end),wavenumber(1:nr_of_gridpoints/2)];
animation_data.grid_range              = grid_range;
animation_data.step_width              = step_width;

myhandles.animation_data = animation_data;

% save guidata
guidata(qs_main,myhandles);

end
