function [potential,EffSlitWidth,EffSlitDist,EffPotWidth]=set_pot(NrOfGridpoints,GridRange,NrOfSlits,SlitWidth,SlitDist,PotHeight,PotWidth,CloseOneSlit)
%  SET_POT   Produces a potential barrier or slit configuration to simulate
%            quantum mechanical scattering processes
%
%  [potential,EffSlitWidth,EffSlitDist,EffPotWidth]= ... 
%             set_pot(NrOfGridpoints,GridRange,NrOfSlits,SlitWidth,SlitDist,PotHeight,PotWidth,CloseOneSlit)
%  INPUT
%    NrOfGridpoints  ...  number of gridpoints
%    GridRange       ...  range of the grid
%    NrOfSlits       ...  desired number of slits in the barrier
%    SlitWidth       ...  desired slit width
%    SlitDist        ...  distance between the centers of two neighbouring slits
%    PotHeight       ...  height of the barrier
%    PotWidth        ...  width of the barrier
%    CloseOneSlit    ...  If this variable ~= 0, half the number of slits
%                         will be closed. With this option it is possible to simulate the
%                         closing of one slit in a double-slit experiment without changing
%                         other values.
%  OUTPUT
%    potential  ...  The produced potential barrier
%    Eff*       ...  Gives the real value of the corresponding physical
%                    values, they are not the same as the input values 
%                    (discrete evaluation)

% checking the number of input arguments and setting defaults
if nargin < 1;
    error('NrOfGridpoints (nr. of datapoints along the axis) is undefined');
end
if nargin < 2;
    error ('GridRange of datapoints is undefined');
end
if nargin < 3;  
   warning('NrOfSlits is undefined; Default:2 set');
   NrOfSlits=2;
end
if nargin < 4;
   warning('SlitWidth is undefined; Default:2 set');
   SlitWidth=2;
end
if nargin < 5;
   warning('SlitDist is undefined; Default: 2*SlitWidth set');
   SlitDist=2*SlitWidth;
end
if nargin < 6;
    warning('PotHeight is undefined; Default:1 set');
    PotHeight=1;
end
if nargin < 7;
    warning('PotWidth is undefined; Default:2 set');
    PotWidth=2;
end

potential = zeros(NrOfGridpoints);
Step_Width = GridRange / NrOfGridpoints;

PotWidthIndex = round(PotWidth / Step_Width);
potential(:,NrOfGridpoints/2 : NrOfGridpoints/2+PotWidthIndex)=PotHeight;
EffPotWidth = PotWidthIndex * Step_Width;

SlitWidthIndex = round((1/2 * SlitWidth)/Step_Width);
EffSlitWidth = 2*SlitWidthIndex*Step_Width;

SlitDistIndex = round((1/2 * SlitDist)/Step_Width);
EffSlitDist = 2*SlitDistIndex * Step_Width;


if (round(NrOfSlits/2) ~= NrOfSlits/2)    %nr of slits is odd
    for j = -floor(NrOfSlits/2) : floor(NrOfSlits/2)
        SlitCenter=NrOfGridpoints/2 + j*(2*SlitWidthIndex+2*SlitDistIndex);
        potential(SlitCenter-SlitWidthIndex:SlitCenter+SlitWidthIndex,:)=0;
    end
end

if (round(NrOfSlits/2)==NrOfSlits/2)    %nr of slits is even
    for j=-(NrOfSlits/2-1):(NrOfSlits/2)
        SlitCenter=(NrOfGridpoints/2-SlitDistIndex-SlitWidthIndex)+j*(2*SlitWidthIndex+2*SlitDistIndex);
        potential(SlitCenter-SlitWidthIndex:SlitCenter+SlitWidthIndex,:)=0;
    end
end

if CloseOneSlit ~= 0;
potential(1:NrOfGridpoints/2,NrOfGridpoints/2:NrOfGridpoints/2+PotWidthIndex)=PotHeight;
end
